unit FlexF;
{
 FlexF - module de flexion pour la langue franaise.
 Copyright (C) 2002 Y. Ouvrard.

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le
modifier conformment aux dispositions de la Licence Publique Gnrale GNU,
telle que publie par la Free Software Foundation ; version 2 de la licence,
ou encore ( votre choix) toute version ultrieure.
Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE
GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION
A UN OBJET PARTICULIER.
Pour plus de dtail, voir la Licence Publique Gnrale GNU .
Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme
temps que ce programme ; si ce n'est pas le cas, crivez  la
Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
Pour tout contact avec les auteurs : yves.ouvrard@collatinus.com

FlexF utilise l'unit jointe TStringManager,
   Class with string functions - by Eric Grobler (egrobler@quoka.com).
   Mes remerciements  Eric.

UTILISATION : deux possibilits (en overload)
1. function Conjugue(
   Inf, P, T, M, V : string; Pr : boolean) : string;

Conjugue le verbe  l'infinitif Inf
     la personne P, au temps T, au mode M et  la voix V.
    Le pronom sujet est retourn si Pr est  true.
    Codes numriques pour la morphologie :
   actif = 1;
   passif = 2;

   ind = 1;
   subj = 2;
   cond = 3;
   imper = 4;
   infin = 5;
   part = 6;
   ger = 7;

   pres = 1;
   fut = 2;
   impf = 3;
   psimple = 4;
   pcompose = 5;
   futant = 6;
   pqp = 7;
   pant = 8;

2. conjugue (Inf : string; P, T, M, V : integer; Pr : boolean) : string;
   Dans cette version, la fonction attend un appel sous forme de chanes :
   exemple
   conjugue ('avoir', 'nous', 'prsent', 'subjonctif', 'actif', true );
   liste des chanes reconnues :

personne : 'je', 'tu', 'il', 'nous', 'vous', 'ils'
temps : 'prsent', 'futur', 'imparfait', 'pass simple',
        'pass [compos]', 'futur antrieur',
        'plus-que-parfait', 'pass antrieur'
mode : 'indicatif', 'subjonctif', 'conditionnel', 'impratif',
       'infinitif', 'participe', 'grondif'
voix : 'actif', 'passif'.


Copyright (C) 2001 Y. Ouvrard.

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le
modifier conformment aux dispositions de la Licence Publique Gnrale GNU,
telle que publie par la Free Software Foundation ; version 2 de la licence,
ou encore ( votre choix) toute version ultrieure.
Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE
GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION
A UN OBJET PARTICULIER.
Pour plus de dtail, voir la Licence Publique Gnrale GNU .
Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme
temps que ce programme ; si ce n'est pas le cas, crivez  la
Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
Pour tout contact avec les auteurs : yves.ouvrard@collatinus.com

FlexF utilise l'unit jointe TStringManager,
   Class with string functions - by Eric Grobler (egrobler@quoka.com).
   Mes remerciements  Eric.

  faire :

   verbe aller, tous modes

   p.s. et pp : le v. simple seoir;

   participe : rgler le pronominal pour le participe prsent.
 }

interface

type

{ TVerbe est la classe de base. J'ai pris le premier groupe comme rfrence,
   puisqu'il reprsente la majorit des verbes}

TVerbe = class
   inf : string;
   constructor create(i : string);
   private
    pronominal : boolean;
    function GetDesFut(index: integer): string;
    function GetDesImpf(index: integer): string;
    function GetDesPsAi(index: integer): string;
    function GetDesPsI(index: integer): string;
    function GetDesPsU(index: integer): string;
    function GetDesSubjPres(index: integer): string;
   public
   // utiles
   function circonflexe : string; virtual;
   // auxiliaire
   function auxiliaire : string;
   // dsinences
   property DesFut[index : integer] : string read GetDesFut;
   property DesImpf[index : integer] : string read GetDesImpf;
   property DesPsAi[index : integer] : string read GetDesPsAi;
   property DesPsI[index : integer] : string read GetDesPsI;
   property DesPsU[index : integer] : string read GetDesPsU;
   property DesSubjPres[index : integer] : string read GetDesSubjPres;
   // radicaux
   function radpres(P : integer) : string; virtual;
   function radfut : string; virtual;
   function radps : string; virtual;

   // manipulations : pronoms-radical-dsinence
   // RD joint radical et dsinence en grant les -e- et les cdilles.
   function Elide ( A, B : string) : string;
   function Pron ( P : integer; F : string; refl : boolean ) : string;
   function RD(R, D : string) : string; virtual;
   function Compose ( A : string; P, T, M, V : integer ) : string;


   // temps
   function IndPres(P : integer) : string; virtual;
   function IndFut (P : integer) : string; virtual;
   function IndImpf ( P : integer) : string; virtual;
   function IndPs ( P : integer ) : string; virtual;
   function SubjPres ( P : integer ) : string; virtual;
   function ImperPres ( P : integer ) : string; virtual;
   function PartPres : string; virtual;
   function PP : string; virtual;

   // procdure de flexion
   function conjugue(P, T, M, V : integer; pr : boolean) : string; overload;
   end;

TVeter = class ( TVerbe )
   function RD ( R, D : string ) : string; override;
   end;

TVettell = class ( TVerbe )
   function RD ( R, D : string ) : string; override;
   end;   

TVebrer = class ( TVerbe )
   function RD ( R, D : string ) : string; override;
   end;

TVyer = class ( TVerbe )
   function RD ( R, D : string ) : string; override;
   end;   

TVavoir = class( TVerbe)
   function radfut : string; override;
   function IndPres (P : integer) : string; override;
   function IndImpf (P : integer) : string; override;
   function IndPs ( P : integer ) : string; override;
   function SubjPres ( P : integer ) : string; override;
   function ImperPres ( P : integer ) : string; override;
   function PartPres : string; override;
   function PP : string; override;
   end;

TVetre = class( TVerbe )
   function RadFut : string; override;
   function RadPs : string; override;
   function IndPres (P : integer) : string; override;
   function IndImpf (P : integer ) : string; override;
   function IndPs ( P : integer ) : string; override;
   function SubjPres ( P : integer ) : string; override;
   function ImperPres ( P : integer ) : string; override;
   function PartPres : string; override;
   function PP : string; override;
  private
   end;

TValler = class ( TVerbe )
   function RadFut : string; override;
   function IndPres ( P : integer ) : string; override;
   function SubjPres ( P : integer ) : string; override;
   end;

TVsst = class( TVerbe )
   function RadPres( P : integer ) : string; override;
   function IndPres (P : integer) : string; override;
   function IndPs ( P : integer ) : string; override;
   function PP : string; override;
   end;

   // hritiers de TVsst

   TVaitre = class( TVsst ) // paratre
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

      TVnaitre = class ( TVaitre )
         function RadPres ( P : integer ) : string; override;
         function RadPs : string; override;
         function IndPs ( P : integer ) : string; override;
         function PP : string; override;
         end;

      TVpaitre = class ( TVaitre )
         // distinguer repatre, qui a un pp.
         function IndPs ( P : integer ) : string; override;
         function RadPs : string; override;
         function PP : string; override;
         end;

   TVboire = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function SubjPres ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVbouillir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      end;

   TVcevoir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadFut : string; override;
      function Radps : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVchoir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function radps : string; override;
      function IndPres ( P : integer ) : string; override;
      function IndPs ( P : integer ) : string; override;
      function SubjPres ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVclore = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function IndPres ( P : integer ) : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVclure = class ( TVsst )
      function radps : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVconfire = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function radps : string; override;
      function PP : string; override;
      end;

   TVcourir = class ( TVsst )
      function RadFut : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVcroire = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVcroitre = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVdevoir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadFut : string; override;
      function Radps : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVdire = class (TVsst)
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPres ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVdormir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      end;

   TVecrire = class (TVsst)
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function PP : string; override;
      end;

   TVfaillir = class ( TVsst )
      function IndPres ( P : integer ) : string; override;
      end;

   TVfaire = class (TVsst)
      function RadPres ( P : integer ) : string; override;
      function RadFut : string; override;
      function RadPs : string; override;
      function IndPres ( P : integer ) : string; override;
      function SubjPres ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVfuir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      end;

   TVgesir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function IndFut ( P : integer ) : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVindre = class( TVsst )
      function RadPres (P : integer) : string; override;
      function RadPs : string; override;
      function PP : string; override;
      end;



   TVir = class( TVsst )
      function RadPres ( P : integer ) : string; override;
      function IndPs (P : integer ) : string; override;
      function PP : string; override;
      end;

   TVlire = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

      TVtaire = class ( TVlire )
         function RadPs : string; override;
         function IndPs ( P : integer ) : string ; override;
         end;

   TVmettre = class( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function PP : string; override;
      end;

   TVmourir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadFut : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVmouvoir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadFut : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVouir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVplaire = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVpleuvoir = class ( TVsst )
      function IndPres ( P : integer ) : string; override;
      function radPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function SubjPres ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVpouvoir = class ( TVsst )
      function RadFut : string; override;
      function RadPs : string; override;
      function IndPres ( P : integer ) : string; override;
      function IndPs ( P : integer ) : string; override;
      function SubjPres ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVpourvoir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVvoir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadFut : string; override;
      function RadPs : string; override;
      function PP : string; override;
      end;

   TVquerir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadFut : string; override;
      function RadPs : string; override;
      function PP : string; override;
      end;

   TVrire = class ( TVsst )
      function RadPs : string; override;
      end;

   TVsavoir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadFut : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function ImperPres ( P : integer ) : string; override;
      function PartPres : string; override;
      function PP : string; override;
      end;

   TVservir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      end;

   TVsoudre = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVsuivre = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      end;

      TVvivre = class ( TVsuivre )
         function RadPs : string; override;
         function IndPs ( P : integer ) : string; override;
         function PP : string; override;
         end;

   TVsurseoir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVvenir = class ( TVsst )
      function radfut : string; override;
      function circonflexe : string; override;
      function RadPres ( P : integer ) : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   // ne pas permuter les 2 suivants
   TVvetir = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVtir = class( TVsst )
      function RadPres ( P : integer ) : string; override;
      function IndPs ( P : integer ) : string; override;
      end;
   // ne pas permuter les 2 prcdents

   TVuire = class ( TVsst )
      function RadPres ( P : integer) : string; override;
      function RadPs : string; override;
      function PP : string; override;
      end;

   TVtraire = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVvaincre = class ( TVsst )
      function RadPres ( P : integer ) : string; override;
      function IndPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function PP : string; override;
      end;

   // fin des hritiers de TVsst

TVxxt = class ( TVsst )
   function RadPs : string; override;
   function IndPres ( P : integer ) : string; override;
   function IndPs ( P : integer ) : string; override;
   end;

   // hritiers de TVxxt

   TVvaloir = class ( TVxxt )
      function RadPres ( P : integer ) : string; override;
      function RadFut : string; override;
      function PP : string; override;
      end;  

   TVvouloir = class ( TVxxt )
      function RadPres ( P : integer ) : string; override;
      function RadFut : string; override;
      function SubjPres ( P : integer ) : string; override;
      function PP : string; override;
      end;

TVdre = class ( TVerbe )
   function IndPres( P : integer ) : string; override;
   function IndPs ( P : integer ) : string; override;
   function PP : string; override;
   end;

   TVasseoir = class ( TVdre )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPres ( P : integer ) : string; override;
      function PP : string; override;
      end;

   TVcoudre = class ( TVdre )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      end;

   TVmoudre = class ( TVdre )
      function RadPres ( P : integer ) : string; override;
      function RadPs : string; override;
      function IndPs ( P : integer ) : string; override;
      end;

   TVprendre = class ( TVdre )
      function radPres ( P : integer ) : string; override;
      function radps : string; override;
      function SubjPres ( P : integer ) : string; override;
      function PP : string; override;
      end;

TVouvrir = class( TVerbe )
      function IndPs ( P : integer ) : string; override;
      function PP : string; override;
   end;

TVbattre = class( TVtir )
   function RadPres ( P : integer ) : string; override;
   function PP : string; override;
   end;

function Conjugue(
   Inf : string; P, T, M, V : integer; Pr : boolean) : string; overload;
function Conjugue(
   Inf, P, T, M, V : string; Pr : boolean) : string; overload;

function PlurielNom ( F : string ) : string;

const
   // chanes de champs morpho
   personne : array[0..6] of shortstring =
     (' ', 'je', 'tu', 'il', 'nous', 'vous', 'ils');

   temps : array[0..8] of shortstring =
     (' ', 'prsent', 'futur', 'imparfait', 'pass simple',
           'pass [compos]', 'futur antrieur',
           'plus-que-parfait', 'pass antrieur' );

   mode : array [0 .. 7] of shortstring =
     (' ', 'indicatif', 'subjonctif', 'conditionnel', 'impratif',
           'infinitif', 'participe', 'grondif');

   voix : array [0 .. 2] of shortstring =
      (' ', 'actif', 'passif');


implementation

uses Classes, chaines;

const
   actif = 1;
   passif = 2;

   ind = 1;
   subj = 2;
   cond = 3;
   imper = 4;
   infin = 5;
   part = 6;
   ger = 7;

   pres = 1;
   fut = 2;
   impf = 3;
   psimple = 4;
   pcompose = 5;
   futant = 6;
   pqp = 7;
   pant = 8;

const
   AuxEtre =
     'advenir aller arriver dcder devenir choir enlaidir entrer intervenir '
   + 'mourir natre obvenir partir parvenir provenir redevenir rentrer '
   + 'reparatre repartir ressortir rester retomber retourner revenir sortir '
   + 'survenir tomber venir';

const eter =
     'acheter racheger bgueter corseter crocheter fileter fureter haleter '
   + 'celer dceler receler ciseler dmanteler carteler encasteler geler '
   + 'dgeler congeler surgeler marteler modeler peler';

var                        
   listeEtre : TStringList;
   listeEter : TStringList;

{ Conjugue }

function Conjugue(Inf : string; P, T, M, V : integer; Pr : boolean) : string;
   var Vb : TVerbe;
       index : integer;
   begin
      result := '?';
      // lexique
      // auxiliaires
      if Inf = 'tre'
         then Vb := TVetre.Create(Inf)
      else if Inf = 'avoir'
         then Vb := TVavoir.Create(Inf)
      else if inf = 'pouvoir'
         then Vb := TVpouvoir.create ( inf )
      else if inf = 'vouloir'
         then Vb := TVvouloir.create ( inf )

      // autres verbes
      else if inf = 'aller'
         then Vb := TValler.create ( inf )
      else if inf = 'asservir'
         then Vb := TVir.create ( inf )
      else if inf = 'faillir'
         then Vb := TVfaillir.Create ( inf )
      else if ( inf = 'frire' ) or ( inf = 'circoncire' )
         then Vb := TVconfire.create ( inf )
      else if inf = 'gsir'
         then Vb := TVgesir.create ( inf )
      else if inf = 'mourir'
         then Vb := TVmourir.create ( inf )
      else if Inf = 'offrir'
         then Vb := TVouvrir.Create(inf)
      else if inf = 'our'
         then Vb := TVouir.create ( inf )
      else if (inf = 'rire') or ( inf = 'sourire' )
         then Vb := TVrire.create ( inf )
      else if (inf = 'voir') or ( inf = 'prvoir' ) or ( inf = 'revoir' )
         then Vb := TVvoir.create ( inf )
      else if ( inf = 'surseoir' )
         then Vb := TVsurseoir.create ( inf )
      else if ( inf = 'taire' )
         then Vb := TVtaire.create ( inf )

      // variantes du premier groupe
      else if ( ListeEter.Find ( inf, index ) )
         or ( IsLast ( 'ecer', inf ) )
         or ( IsLast ( 'emer', inf ) )
         or ( IsLast ( 'ener', inf ) )
         or ( IsLast ( 'eper', inf ) )
         or ( IsLast ( 'erer', inf ) )
         or ( IsLast ( 'eser', inf ) )
         or ( IsLast ( 'ever', inf ) )
         or ( IsLast ( 'evrer', inf ) )
         then Vb := TVEter.create ( inf )

      else if
            ( IsLast ( 'brer', inf ) )
         or ( IsLast ( 'cer', inf ) )
         or ( IsLast ( 'cher', inf ) )
         or ( IsLast ( 'crer', inf ) )
         or ( IsLast ( 'der', inf ) )
         or ( IsLast ( 'gler', inf ) )
         or ( IsLast ( 'gner', inf ) )
         or ( IsLast ( 'grer', inf ) )
         or ( IsLast ( 'guer', inf ) )
         or ( IsLast ( 'ler', inf ) )
         or ( IsLast ( 'mer', inf ) )
         or ( IsLast ( 'ner', inf ) )
         or ( IsLast ( 'per', inf ) )
         or ( IsLast ( 'quer', inf ) )
         or ( IsLast ( 'rer', inf ) )
         or ( IsLast ( 'ser', inf ) )
         or ( IsLast ( 'ter', inf ) )
         or ( IsLast ( 'trer', inf ) )
         or ( IsLast ( 'vrer', inf ) )
         or ( IsLast ( 'yer', inf ) )
        then Vb := TVebrer.create ( inf )

      else if (IsLast ( 'eler', inf ) ) or (IsLast ( 'eter', inf ) )
         then Vb := TVettell.create ( inf )

      else if (IsLast ( 'yer', inf ) )
         then Vb := TVyer.create ( inf )

      // dsinence d'infinitif
      else if (IsLast('er', inf))
         then Vb := TVerbe.Create(Inf)
      // natre, patre et paratre : ne pas sparer
      else if IsLast('natre', inf)
         then Vb := TVnaitre.create ( inf )
      else if IsLast( 'patre', inf )
         then Vb := TVpaitre.create ( inf)
      else if IsLast('atre', inf)
         then Vb := TVaitre.Create( inf )
      // ne pas sparer les 3 prcdentes

      else if IsLast( 'asseoir', inf )
         then Vb := TVasseoir.create ( inf )
      else if IsLast('battre', inf)
         then Vb := TVbattre.Create(Inf)
      else if IsLast ( 'boire', inf )
         then Vb := TVboire.create ( inf )
      else if IsLast ( 'bouillir', inf )
         then Vb := TVbouillir.Create ( inf )
      else if IsLast ( 'cevoir', inf )
         then Vb := TVcevoir.create ( inf )
      else if IsLast ( 'choir', inf )
         then Vb := TVchoir.create ( inf )
      else if IsLast ( 'clore', inf )
         then Vb := TVclore.create ( inf )
      else if IsLast ( 'clure', inf )
         then Vb := TVclure.create ( inf )
      else if IsLast ( 'confire', inf )
         then Vb := TVconfire.create ( inf )
      else if IsLast('courir', inf)
         then Vb := TVcourir.Create(inf)
      else if IsLast ( 'croire', inf )
         then Vb := TVcroire.create ( inf )
      else if IsLast ( 'crotre', inf )
         then Vb := TVcroitre.create ( inf )
      else if IsLast ( 'cueillir', inf )
         then Vb := TVerbe.create( inf )
      else if IsLast ( 'devoir', inf )
         then Vb := TVdevoir.create ( inf ) 
      else if IsLast('dire', inf)
         then Vb := TVdire.Create(inf)
      else if IsLast( 'dormir', inf )
         then Vb := TVdormir.create ( inf )
      else if IsLast('faire', inf)
         then Vb := TVfaire.Create(inf)
      else if IsLast('crire', inf) or IsLast('scrire', inf)
         then Vb := TVecrire.Create(inf)
      else if IsLast ( 'fuir', inf )
         then Vb := TVfuir.Create ( inf )
      else if IsLast ( 'lire', inf )
         then Vb := TVlire.create ( inf )
      else if IsLast ( 'mouvoir', inf )
         then Vb := TVmouvoir.create ( inf )
      else if IsLast('ouvrir', inf)
         then Vb := TVouvrir.Create(inf)
      else if IsLast('mettre', Inf)
         then Vb := TVmettre.Create(Inf)
      else if IsLast ( 'plaire', Inf )
         then Vb := TVplaire.create ( inf ) 
      else if IsLast ( 'pleuvoir', inf )
         then Vb := TVpleuvoir.create ( inf )
      else if IsLast('pourvoir', inf )
         then Vb := TVpourvoir.create ( inf )
      else if IsLast('prendre', Inf)
         then Vb := TVprendre.Create( inf )
      else if IsLast('qurir', inf)
         then Vb := TVquerir.create ( inf )
      else if IsLast('saillir', inf)
         then Vb := TVerbe.create ( inf )
      else if IsLast('savoir', inf)
         then Vb := TVsavoir.create ( inf )
      else if IsLast('servir', inf)
         then Vb := TVservir.create ( inf )
      else if IsLast ( 'soudre', inf )
         then Vb := TVsoudre.create ( inf )
      else if IsLast ( 'suivre', inf )
         then Vb := TVsuivre.create ( inf )
      else if IsLast('traire', inf)
         then Vb := TVtraire.create( inf )
      else if IsLast( 'vaincre', inf )
         then Vb := TVvaincre.create( inf )
      else if IsLast ( 'venir', inf ) or IsLast ( 'tenir', inf )
         then Vb := TVvenir.create ( inf )
      else if IsLast('uire', inf)
         then Vb := TVuire.Create( inf )
      else if IsLast ('valoir', inf)
         then Vb := TVvaloir.create ( inf )
      else if IsLast ('vivre', inf )
         then Vb := TVvivre.create ( inf )

      // -dre
      else if IsLast('indre', inf)
         then Vb := TVindre.Create(inf)
      else if IsLast('coudre', inf)
         then Vb := TVcoudre.create ( inf )
      else if IsLast ( 'moudre', inf )
         then Vb := TVmoudre.create ( inf )
      else if IsLast('dre', inf)
         then Vb := TVDre.Create(Inf)

      // -tir
      else if IsLast('vtir', inf)
         then Vb := TVvetir.Create ( inf )
      else if IsLast('tir', inf)
         then Vb := TVtir.Create(inf)

      // 2me groupe
      else if IsLast('ir', inf)
         then Vb := TVir.Create(inf)

      else exit;
      result := Vb.Conjugue(P, T, M, V, Pr);
      Vb.Free;
   end;


function Conjugue(
   Inf, P, T, M, V : string; Pr : boolean) : string; overload;
   function num ( C : string; A : Array of shortstring ) : integer;
      var i : integer;
   begin
      result := 0;
      for i := low(A) to high(A)
         do if A[i] = C then
            begin
               result := i;
               break;
            end;
   end;
begin
   result := conjugue (
      inf, Num(P, personne), num(T, temps), num(M, mode), num(V, voix), Pr);
end;
{ TVerbe }

function TVerbe.auxiliaire: string;
   var i : integer;
begin
if ListeEtre.Find ( inf, i )
     then result := 'tre'
     else result := 'avoir';
end;

function TVerbe.circonflexe: string;
   var d : char;
begin
   result := IndPs ( 2 ) ;
   result := DelEnd ( result, 1 );
   case LastChar(result) of
      'a' : d := '';
      'i', '' : d := '';
      'u', '' : d := '';
      else d := #0;
      end;
   result := DelEnd ( result, 1 ) + d;
end;

function TVerbe.Compose(A: string; P, T, M, V : integer): string;
   var partp : string;
begin
   partp := PP;
   if partp > '' then
      begin
         result := FlexF.conjugue (
           A, P, T, M, V, false ) + ' ' + partp;
         if (P > 3) and ( A = 'tre' ) and (LastChar ( result ) <> 's')
            then result := result + 's';
      end
      else result := '';
end;

function TVerbe.conjugue(P, T, M, V: integer; pr: boolean): string;
   var aux : shortstring;
       PPPP : boolean ;
begin
// vrifier la validit du champ P(ersonne)

// pronominal
   if (pos ('s''', inf) = 1) or (pos ('se ', inf) = 1) then
      begin
        pronominal := true;
        aux := 'tre';
      end
      else
      begin
         pronominal := false;
         aux := auxiliaire;
      end;

// prvoir le PPPP (part. prsent ou pass pronominal)
   PPPP := ( M = part ) and ( pronominal = true );

// liminer le pronom sujet   
   case M of
      imper, part, infin : pr := false;
      end;

// 1. voix
case V of
   actif :
     case M of // actif
        ind :  // indicatif actif
           case T of
              pres: result := IndPres(P);
              fut: result := IndFut ( P );
              impf: result := IndImpf ( P );
              psimple: result := IndPs ( P);
              else result := compose ( aux, P, T - 4, ind, actif );
              end;
        subj: // subjonctif actif
           case T of
              pres : result := SubjPres ( P );
              impf :
                 if RadPs > '' then
                    begin
                       if P = 3
                          then result := circonflexe  + 't'
                          else result := IndPs ( 2 ) + 's' + DesSubjPres[P]
                    end
                    else result := '';
              pcompose : result := compose ( aux, P, pres, subj, actif );
              pqp : result := compose ( aux, P, impf, subj, actif );
              else result := 'erreur';
              end;
        cond:
           case T of
              pres : // calcul d'aprs le futur
                result := RD ( RadFut, DesImpf[P] );
              pcompose : result := compose ( aux, P, pres, cond, actif );
              else result := 'erreur';
              end;
        imper:
           case P of
              1, 3, 6 : result := 'erreur'
              else
              case T of
                 pres : result := ImperPres ( P );
                 pcompose : result := compose ( aux, P, pres, imper, actif );
                 end;
              end;
        infin :
           case T of
              pres : result := inf;
              pcompose : result := compose ( aux, P, pres, infin, actif );
              end;
        part :
           case T of
              pres :
                 result := PartPres ;
              pcompose : result := compose ( aux, P, pres, part, actif );
              end;
        end;
   passif :
       if (PP > '') and (aux = 'avoir') then
          result := compose ( 'tre', P, T, M, actif );
   end;
// pronom
   // liminer le pronom de l'entre
   result := Del( 's''', result);
   if IsFirst ( 'se ', result )
      then result := Cut ( result, 1, 3 );
   if pr then result :=  Pron ( P, result, pronominal )
   else if PPPP then result := elide ( 'se', result );
end;

constructor TVerbe.create(i: string);
begin
   inf := i;
end;

function TVerbe.Elide(A, B: string): string;
   const  voyelles = 'aeiouy';
begin
   if (LastChar ( A ) = 'e' ) and (pos( B[1], voyelles ) > 0)
      then result := DelEnd ( A, 1 ) + '''' + B
      else result := A + ' ' + B;
end;

function TVerbe.GetDesFut(index: integer): string;
   const D : array [1..6] of shortstring =
     ('ai', 'as', 'a', 'ons', 'ez', 'ont');
begin
   result := D[index];
end;

function TVerbe.GetDesImpf(index: integer): string;
   const D : array [1..6] of shortstring =
     ('ais', 'ais', 'ait', 'ions', 'iez', 'aient');
begin
   result := D[index];
end;

function TVerbe.GetDesPsAi(index: integer): string;
   const D : array[1..6] of shortstring =
      ('ai', 'as', 'a', 'mes', 'tes', 'rent');
begin
   result := D[index];
end;

function TVerbe.GetDesPsI(index: integer): string;
   const D : array[1..6] of shortstring =
      ('is', 'is', 'it', 'mes', 'tes', 'irent');
begin
   result := D[index];
end;

function TVerbe.GetDesPsU(index: integer): string;
   const D : array[1..6] of shortstring =
      ('us', 'us', 'ut', 'mes', 'tes', 'urent');
begin
   result := D[index];
end;

function TVerbe.GetDesSubjPres(index: integer): string;
   const D : array [1..6] of shortstring =
      ('e', 'es', 'e', 'ions', 'iez', 'ent');
begin
   result := D[index];
end;

function TVerbe.ImperPres(P: integer): string;
begin
   case P of
      2 :
         begin
            result := IndPres ( 2 );
            if (IsLast( 'er', inf ))
               then result := DelEnd ( result, 1 );
         end;
      4, 5 : result := IndPres ( P );
      else result := 'erreur';
      end;
end;

function TVerbe.IndFut(P: integer): string;
begin
   result := RD (RadFut, DesFut[P]);
end;

function TVerbe.IndImpf(P: integer): string;
begin
   result := RD (RadPres ( 4 ), DesImpf[P]);
end;

function TVerbe.IndPres(P: integer): string;
   var D : string;
begin
   Case P of
      1, 3 : D := 'e';
      2 : D := 'es';
      4 : D := 'ons';
      5 : D := 'ez';
      6 : D := 'ent';
      end;
   Result := RD(RadPres(P), D);
end;

function TVerbe.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsAi[P] );
end;

function TVerbe.PartPres : string;
begin
   result := RadPres ( 4 ) + 'ant';
end;

function TVerbe.PP : string;
begin
   result := RadPres ( 1 ) + '';
end;

function TVerbe.Pron(P : integer; F: string; refl: boolean): string;
   const
      PR : array [1..6] of shortstring =
         ('me', 'te', 'se', 'nous', 'vous', 'se');
begin
   result := personne[P] ;
   if refl then result := result + ' ' + PR[P];

   // lision
   result := elide ( result, F );
end;

function TVerbe.radfut : string;
begin
   if IsLast ('r', inf)
      then result := inf
      else result := DelEnd (inf, 1 );
end;

function TVerbe.radpres (P : integer): string;
begin
   result := DelEnd(inf, 2);
end;

function TVerbe.radps: string;
begin
   result := DelEnd ( inf, 2 );
end;

function TVerbe.RD(R, D: string): string;
begin
   if (D > '') and (pos(D[1], 'oau') > 0) then
      begin
         if LastChar(R) = 'c'
            then R[length(R)] := ''
         else if LastChar(R) = 'g'
            then R := R + 'e';
      end;
   result := R + D;
end;

function TVerbe.SubjPres(P: integer): string;
begin
   // result := RadPres ( 6 ) + DesSubjPres[P];
   case P of
      4, 5 : result := RadPres ( 4 ) + DesSubjPres[P];
      else result := RadPres ( 6 ) + DesSubjPres[P];
      end;
end;

{ TVDre }

function TVdre.IndPres(P: integer): string;
   var D : string;
begin
   result := '';
   case P of
      1, 2 : D := 's';
      3 : D := '';
      else Result := inherited IndPres ( P );
      end;
   if result = ''
      then Result := RD(RadPres(P), D);
end;

function TVdre.IndPs(P: integer): string;
begin
   result := RD ( radPs, DesPsI[P]);
end;

function TVdre.PP: string;
begin
   result := RadPs + 'u';
end;

{ TVindre }

function TVindre.PP: string;
begin
   result := IndPres ( 3 );
end;

function TVindre.RadPres(P: integer): string;
begin
   if p < 4
      then Result := DelEnd(inf, 3)
      else Result := DelEnd(inf, 4) + 'gn';
end;

function TVindre.RadPs: string;
begin
   result := RadPres ( 4 );
end;

{ TVetre }

function TVetre.ImperPres(P: integer): string;
begin
   case P of
      2, 4, 5 : result := SubjPres ( P );
      else result := 'erreur';
      end;
end;

function TVetre.IndImpf ( P : integer ) : string;
begin
   result := RD ( 't', DesImpf[P] );
end;

function TVetre.IndPres(P: integer): string;
   const D : array[1..6] of shortstring =
     ('suis', 'es', 'est', 'sommes', 'tes', 'sont');
begin
   result := D[P];
end;

function TVetre.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsU[P] );
end;

function TVetre.PartPres : string;
begin
   result := 'tant';
end;

function TVetre.PP: string;
begin
   result := 't';
end;

function TVetre.RadFut: string;
begin
   result := 'ser';
end;

function TVetre.RadPs: string;
begin
   result := 'f';
end;

function TVetre.SubjPres(P: integer): string;
   const F : array [1..6] of shortstring =
      ('sois', 'sois', 'soit', 'soyons', 'soyez', 'soient');
begin
   result := F[P];
end;

{ TVavoir }

function TVavoir.ImperPres(P: integer): string;
   const F : array[1..6] of shortstring =
      ('', 'aie', '', 'ayons', 'ayez', '');
begin
   result := F[P];
end;

function TVavoir.IndImpf(P: integer): string;
begin
   result := RD ( 'av', DesImpf[P] );
end;

function TVavoir.IndPres(P: integer): string;
   const D : array[1..6] of shortstring =
     ('ai', 'as', 'a', 'avons', 'avez', 'ont');
begin
   result := D[P];
end;

function TVavoir.IndPs(P: integer): string;
begin
   result := RD ( 'e', DesPsU[P] )
end;

function TVavoir.PartPres : string;
begin
   result := 'ayant';
end;

function TVavoir.PP: string;
begin
   result := 'eu';
end;

function TVavoir.radfut: string;
begin
   result := 'aur';
end;

function TVavoir.SubjPres(P: integer): string;
   const F : array[1..6] of shortstring =
     ('aie', 'aies', 'ait', 'ayons', 'ayez', 'aient');
begin
   result := F[P];
end;

{ TVprendre }

function TVprendre.PP: string;
begin
   result := IndPs ( 1 );
end;

function TVprendre.RadPres(P: integer): string;
begin
   if P > 3
      then result := DelEnd(inf, 3)
      else result := DelEnd( inf, 2);
end;

function TVprendre.radps: string;
begin
   result := DelEnd ( inf, 5 );
end;

function TVprendre.SubjPres(P: integer): string;
   var D, R : shortstring;
begin
   D := DesSubjPres[P];
   R := RadPres ( 4 );
   if D[1] = 'e'
      then R := R + 'n';
   result := R + D;
end;

{ TVmettre }

function TVmettre.PP: string;
begin
   result := RadPs + 'is';
end;

function TVmettre.RadPres(P: integer): string;
begin
   case P of
      1,2 : result := DelEnd(inf, 3);
      3 : result := DelEnd(inf, 4);
      else result := DelEnd(inf, 2);
      end;
end;

function TVmettre.RadPs: string;
begin
   result := DelEnd ( inf, 5 ) ;
end;

{ TVsst }

function TVsst.IndPres(P: integer): string;
   var D : string;
begin
   result := '';
   case P of
     1, 2 : D := 's';
     3 : D := 't';
     else result := inherited IndPres( P );
     end;
   if result = ''
      then result := RD(RadPres(P), D);
end;

function TVsst.IndPs(P: integer): string;
begin
   Result := RD ( radps, DesPsI[P] );
end;

function TVsst.PP: string;
begin
   result := RadPs + 'i';
end;

function TVsst.RadPres(P: integer): string;
begin
   result := DelEnd(inf, 2);
end;

{ TVuire }

function TVuire.PP: string;
begin
   if (inf = 'luire') or (inf = 'nuire')
      then result := DelEnd ( inf, 2 )
      else result := IndPres ( 3 ); 
end;

function TVuire.RadPres(P: integer): string;
begin
   Result := inherited RadPres(P);
   if P > 3
      then Result := result + 's';
end;

function TVuire.RadPs: string;
begin
   result := RadPres ( 4 );
end;

{ TVtir }

function TVtir.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsI[P] );
end;

function TVtir.RadPres(P: integer): string;
begin
   if P < 4
      then result := DelEnd( inf, 3)
      else result := inherited RadPres( P ) ;   
end;

{ TVbattre }

function TVbattre.PP: string;
begin
   result := radPs + 'u';
end;

function TVbattre.RadPres(P: integer): string;
begin
   if P = 3
      then result := DelEnd(inf, 4)
   else result := inherited RadPres( P );
end;

{ TVir }

function TVir.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsI[P] ) ;
end;

function TVir.PP: string;
begin
   result := DelEnd ( inf, 2 ) + 'i';
end;

function TVir.RadPres(P: integer): string;
begin
  result := DelEnd(inf, 1);
  if P > 3
     then result := result + 'ss';
end;

{ TVfaire }

function TVfaire.IndPres(P: integer): string;
begin
   case P of
      5 : result := DelEnd(inf, 2) + 'tes';
      6 : result := DelEnd(inf, 4) + 'ont';
      else result := inherited IndPres(P);
      end;
end;

function TVfaire.PP: string;
begin
   result := IndPres ( 3 );
end;

function TVfaire.RadFut: string;
begin
   result := DelEnd ( inf, 4 ) + 'er';
end;

function TVfaire.RadPres(P: integer): string;
begin
   result := inherited RadPres(P);
   if P > 3
      then result := result + 's'; 
end;

function TVfaire.RadPs: string;
begin
   result := DelEnd ( inf, 4 );
end;

function TVfaire.SubjPres(P: integer): string;
begin
   result := DelEnd ( inf, 3 ) + 'ss' + DesSubjPres[P];   
end;

{ TVaitre }

function TVaitre.IndPs(P: integer): string;
begin
   Result := RD ( radps, DesPsU[P] );
end;

function TVaitre.PP: string;
begin
   result := RadPs + 'u';
end;

function TVaitre.RadPres(P: integer): string;
begin
   case P of
      1, 2 : result := DelEnd (inf, 4) + 'i';
      3 : result := DelEnd ( inf, 3 );
      else result := DelEnd( inf, 4 ) + 'iss';
      end;
end;

function TVaitre.RadPs: string;
begin
   result := DelEnd ( inf, 5 );
end;

{ TVdire }

function TVdire.IndPres(P: integer): string;
begin
   if (inf = 'dire') and (P = 5)
      then result := 'dites'
      else result := inherited IndPres( P );
end;

function TVdire.PP: string;
begin
   result := DelEnd ( inf, 2 ) + 't';
end;

function TVdire.RadPres(P: integer): string;
begin
   case P of
      1..3 : result := DelEnd(inf, 2);
      else result := DelEnd( inf, 2 ) + 's';
      end;
end;

function TVdire.RadPs: string;
begin
   result := DelEnd ( inf, 3 );
end;

{ TVecrire }

function TVecrire.PP: string;
begin
   result := DelEnd ( inf, 2 ) + 't';
end;

function TVecrire.RadPres(P: integer): string;
begin
   Result := DelEnd( inf, 2 );
   if P > 3
      then Result := result + 'v';
end;

function TVecrire.RadPs: string;
begin
   result := RadPres ( 4 );
end;

{ TVquerir }

function TVquerir.PP: string;
begin
   result := IndPs ( 1 );
end;

function TVquerir.RadFut: string;
begin
   result := DelEnd ( inf, 4 ) + 'err';
end;

function TVquerir.RadPres(P: integer): string;
begin
   result := DelEnd( inf, 4 );
   case P of
      1..3 : result := result + 'ier';
      4, 5 : result := result + 'r';
      6 : result := result + 'ir';
      end;
end;

function TVquerir.RadPs: string;
begin
   Result := DelEnd ( inf, 4 );
end;

{ TVtraire }

function TVtraire.IndPs(P: integer): string;
begin
   result := '';
end;

function TVtraire.PP: string;
begin
   result := IndPres ( 3 );
end;

function TVtraire.RadPres(P: integer): string;
begin
   case P of
      4, 5 : result := DelEnd ( inf, 3 ) + 'y';
      else result := DelEnd(inf, 2);
      end;
end;

{ TVpouvoir }

function TVpouvoir.IndPres(P: integer): string;
   const D : array[1..6] of shortstring =
     ('peux', 'peux', 'peut', 'pouvons', 'pouvez', 'peuvent');
begin
   result := D[P];
end;

function TVpouvoir.IndPs(P: integer): string;
begin
   result := RD ( RadPs, DesPSU[P] );
end;

function TVpouvoir.PP: string;
begin
  result := 'pu';
end;

function TVpouvoir.RadFut: string;
begin
   result := DelEnd ( inf, 4 ) + 'rr';
end;

function TVpouvoir.RadPs: string;
begin
   result := 'p';
end;

function TVpouvoir.SubjPres(P: integer): string;
begin
   result := 'puiss' + DesSubjPres[P];
end;

{ TVvouloir }

function TVvouloir.PP: string;
begin
   result := RadPs + 'u';
end;

function TVvouloir.RadFut: string;
begin
   result := DelEnd ( inf, 4 ) + 'dr';
end;

function TVvouloir.RadPres(P: integer): string;
begin
   case P of
      1..3 : result := DelEnd ( inf, 6 ) + 'eu';
      4, 5 : result := DelEnd ( inf, 3 );
      6 : result := DelEnd ( inf, 6 ) + 'eul';
      end;
end;

function TVvouloir.SubjPres(P: integer): string;
   var R, D : shortstring;
begin
   D := DesSubjPres[P];
   if D[1] = 'e'
      then R := 'veuill'
      else R := 'voul';
   result := R + D;
end;

{ TVvetir }

function TVvetir.PP: string;
begin
   result := RadPs + 'u';
end;

function TVvetir.RadPres(P: integer): string;
begin
   if P = 3
      then result := DelEnd ( inf, 3 )
      else result := inherited RadPres ( P );
end;

{ TVbouillir }

function TVbouillir.RadPres(P: integer): string;
begin
   if P > 3
      then result := DelEnd( inf, 2)
      else result := DelEnd ( inf, 5 );
end;

{ TVdormir }

function TVdormir.RadPres(P: integer): string;
begin
   if P > 3
      then result := DelEnd( inf, 2 )
      else result := DelEnd ( inf, 3 );
end;

{ TVmourir }

function TVmourir.IndPs(P: integer): string;
begin
   result := RD ( radPs, DesPsU[P] );
end;

function TVmourir.PP: string;
begin
   result := 'mort';
end;

function TVmourir.RadFut: string;
begin
   result := DelEnd ( inf, 2 ) + 'r';
end;

function TVmourir.RadPres(P: integer): string;
begin
   case P of
     4, 5 : result := 'mour';
     else result := 'meur';
     end;
end;

{ TVservir }

function TVservir.RadPres(P: integer): string;
begin
   case P of
      1..3 : result := DelEnd( inf, 3 );
      else result := DelEnd ( inf, 2 );
      end;
end;

{ TVfuir }

function TVfuir.RadPres(P: integer): string;
begin
   Case P of
      4, 5 : result := DelEnd ( inf, 2 ) + 'y';
      else result := DelEnd ( inf, 1 );
      end;
end;

{ TVouir }

function TVouir.IndPs(P: integer): string;
   const F : array [1..6] of shortstring =
   ('ous', 'ous', 'out', 'oumes', 'outes', 'ourent');
begin
   result := F[P];
end;

function TVouir.PP: string;
begin
   result := 'ou';
end;

function TVouir.RadPres(P: integer): string;
begin
   case P of
      4, 5 : result := 'oy';
      else result := 'oi';
      end;
end;

{ TVgesir }

function TVgesir.IndFut(P: integer): string;
begin
   result := '';
end;

function TVgesir.IndPs(P: integer): string;
begin
   result := '';
end;

function TVgesir.PP: string;
begin
   result := '';
end;

function TVgesir.RadPres(P: integer): string;
begin
   Case P of
      1, 2 : result := 'gi';
      3 : result := 'g';
      else result := 'gis';
      end;
end;

{ TVcevoir }

function TVcevoir.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsU[P] );
end;

function TVcevoir.PP: string;
begin
   result := RD ( radPs, 'u' );
end;

function TVcevoir.RadFut: string;
begin
   result := DelEnd ( inf, 3 ) + 'r';
end;

function TVcevoir.RadPres(P: integer): string;
begin
   case P of
     1..3 : result := DelEnd ( inf, 6 ) + 'oi';
     4, 5 : result := DelEnd ( inf, 3 );
     6 : result := DelEnd ( inf, 6 ) + 'oiv';
     end;
end;

function TVcevoir.Radps: string;
begin
   result := DelEnd ( inf, 6 ) + 'c';
end;

{ TVvoir }

function TVvoir.PP: string;
begin
   result := RadPs + 'u';
end;

function TVvoir.RadFut: string;
begin
   if inf = 'voir'
      then result := 'verr'
      else result := inf;
end;

function TVvoir.RadPres(P: integer): string;
begin
   case P of
      4, 5 : result := DelEnd ( inf, 2 ) + 'y';
      else result := DelEnd ( inf, 1 )
      end;
end;

function TVvoir.RadPs: string;
begin
   result := DelEnd ( inf, 3 );
end;

{ TVdevoir }

function TVdevoir.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsU[P] );
end;

function TVdevoir.PP: string;
begin
   result := RadPs + '';
end;

function TVdevoir.RadFut: string;
begin
   result := DelEnd ( inf, 3 ) + 'r';
end;

function TVdevoir.RadPres(P: integer): string;
begin
   case P of
      1..3 : result := DelEnd ( inf, 5 ) + 'oi';
      4, 5 : result := DelEnd ( inf, 3 );
      6 : result := DelEnd ( inf, 5 ) + 'oiv';
      end;
end;

function TVdevoir.Radps: string;
begin
   result := DelEnd ( inf, 5 );
end;

{ TVfaillir }

function TVfaillir.IndPres(P: integer): string;
   const D : array [1..6] of ShortString =
      ( 'faux', 'faux', 'faut', 'faillons', 'faillez', 'faillent' );
begin
   result := D[P];
end;

{ TVpourvoir }

function TVpourvoir.IndPs(P: integer): string;
begin
   result := RD ( RadPs, DesPsU[P] );
end;

function TVpourvoir.PP: string;
begin
   result := RadPs + 'u';
end;

function TVpourvoir.RadPres(P: integer): string;
begin
   Case P of
      4, 5 : result := DelEnd ( inf, 2 ) + 'y';
      else result := DelEnd( inf, 1);
      end;

end;

function TVpourvoir.RadPs: string;
begin
   result := DelEnd ( inf, 3 );
end;

{ TVsavoir }

function TVsavoir.ImperPres(P: integer): string;
   const D : array [1..6] of shortstring =
      ('', 'e', '', 'ons', 'ez', '');
begin
   if D[P] > ''
      then result := DelEnd ( inf, 4 ) + 'ch' + D[P]
      else result := '';
end;

function TVsavoir.IndPs(P: integer): string;
begin
   result := RD ( radPs, DesPsU[P] );
end;

function TVsavoir.PartPres : string;
begin
   result := DelEnd ( inf, 4 ) + 'chant';
end;

function TVsavoir.PP: string;
begin
   result := RadPs + 'u';
end;

function TVsavoir.RadFut : string;
begin
   result := DelEnd ( inf, 4 ) + 'ur';
end;

function TVsavoir.RadPres(P: integer): string;
begin
   if P < 4
      then result := DelEnd( inf, 4 ) + 'i'
      else result := DelEnd( inf, 3 );
end;

function TVsavoir.RadPs: string;
begin
   result := DelEnd ( inf, 5 );
end;

{ TVmouvoir }

function TVmouvoir.IndPs(P: integer): string;
begin
   result := RD ( radPs, DesPsU[P] );
end;

function TVmouvoir.PP: string;
begin
   if inf = 'mouvoir'
      then result := 'm'
      else result := radPs + 'u';
end;

function TVmouvoir.RadFut: string;
begin
   result := DelEnd ( inf, 3 ) + 'r';
end;

function TVmouvoir.RadPres(P: integer): string;
begin
   case P of
     1..3 : result := DelEnd ( inf, 6 ) + 'eu';
     4, 5 : result := DelEnd ( inf, 3 );
     else result := DelEnd ( inf, 6 ) + 'euv';
     end;
end;

function TVmouvoir.RadPs: string;
begin
   result := DelEnd ( inf, 6 );
end;

{ TVpleuvoir }

function TVpleuvoir.IndPres(P: integer): string;
begin
   Case P of
      3 : result := DelEnd ( inf, 4 ) + 't';
      6 : result := DelEnd ( inf, 3 ) + 'ent';
      else result := '';
      end;
end;

function TVpleuvoir.IndPs(P: integer): string;
begin
   case P of
      3, 6 : result := RD ( radps, DesPsU[P] );
      else result := '';
      end;
end;

function TVpleuvoir.PP: string;
begin
   result := RadPs + 'u';
end;

function TVpleuvoir.radPs: string;
begin
   result := DelEnd ( inf, 6 );
end;

function TVpleuvoir.SubjPres(P: integer): string;
begin
   case P of
      3, 6 : result := DelEnd ( inf, 3 ) + DesSubjPres[P];
      else result := '';
      end;
end;

{ TVxxt }

function TVxxt.IndPres(P: integer): string;
begin
   if P < 3
      then result := RadPres ( P ) + 'x'
      else result := inherited IndPres ( P );
end;

function TVxxt.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsU[P] );
end;

function TVxxt.RadPs: string;
begin
   result := DelEnd ( inf, 3 );
end;

{ TVvaloir }

function TVvaloir.PP: string;
begin
   result := RadPs + 'u';
end;

function TVvaloir.RadFut: string;
begin
   result := DelEnd ( inf, 4 ) + 'udr';
end;

function TVvaloir.RadPres(P: integer): string;
begin
   if P < 4
      then result := DelEnd ( inf, 4 ) + 'u'
      else result := DelEnd ( inf, 3 );
end;

{ TVasseoir }

function TVasseoir.IndPres(P: integer): string;
   var D : shortString;
begin
  result := '';
  Case P of
     1, 2 : D := 'ds';
     3 : D := 'd';
     else result := inherited IndPres ( P );
     end;
  if result = ''
     then result := RD ( RadPres( P ), D );
end;

function TVasseoir.PP: string;
begin
   result := radPs + 'is';
end;

function TVasseoir.RadPres(P: integer): string;
begin
   if P < 4
      then result := DelEnd( inf , 4 ) + 'ie'
      else result := DelEnd ( inf, 3 ) + 'y';
end;

{ TVsurseoir }

function TVsurseoir.IndPs(P: integer): string;
begin
   if inf = 'seoir'
      then result := ''
      else result := inherited IndPs ( P );
end;

function TVsurseoir.PP: string;
begin
   result := IndPs ( 1 );
end;

function TVsurseoir.RadPres(P: integer): string;
begin
   result := 'surso';
   case P of
      4, 5 : result := result + 'y';
      else result := result + 'i';
      end;
end;

function TVasseoir.RadPs: string;
begin
   result := DelEnd ( inf, 4 );
end;

function TVsurseoir.RadPs: string;
begin
   result := DelEnd ( inf, 4 );
end;

{ TVchoir }

function TVchoir.IndPres(P: integer): string;
begin
   case P of
      4, 5 :
         begin
            if inf = 'dchoir'  then
               begin
                 result := DelEnd( inf, 2 );
                 if P = 4 then result := result + 'yons'
                    else result := result + 'yez';
               end
            else result := '';
         end;
      else result := inherited IndPres ( P );
      end;
end;

function TVchoir.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsU[P] );
end;

function TVchoir.PP: string;
begin
   result := radPs + 'u';
end;

function TVchoir.RadPres(P: integer): string;
begin
   result := DelEnd( inf, 1 );
end;

function TVchoir.radps: string;
begin
   result := DelEnd ( inf, 3 );
end;

function TVchoir.SubjPres(P: integer): string;
begin
   if inf = 'choir'
      then result := inherited SubjPres ( P )
      else result := '';
end;

{ TVvaincre }

function TVvaincre.IndPres(P: integer): string;
begin
   if P = 3 then result := RadPres ( 3 )
      else result := inherited IndPres ( P );
end;

function TVvaincre.PP: string;
begin
   result := DelEnd ( inf, 3 ) + 'u';
end;

function TVvaincre.RadPres(P: integer): string;
begin
   if P < 4
      then result := DelEnd ( inf, 2 )
      else result := DelEnd ( inf, 3 ) + 'qu';
end;

function TVvaincre.RadPs: string;
begin
   result := RadPres ( 4 );
end;

{ TVplaire }

function TVplaire.IndPs(P: integer): string;
begin
   result := RD ( RadPs, DesPsU[P] );
end;

function TVplaire.PP: string;
begin
   result := RadPs + 'u';
end;

function TVplaire.RadPres(P: integer): string;
begin
   Case P of
     1, 2 : result := DelEnd ( inf, 2 );
     3 : result := DelEnd ( inf, 3 ) + '';
     else result := DelEnd ( inf, 2 ) + 's';
     end;
end;

function TVplaire.RadPs : string;
begin
  result := DelEnd ( inf, 4 );
end;

{ TVnaitre }

function TVnaitre.IndPs(P: integer): string;
begin
   result := RadPs + DesPsI[P];
end;

function TVnaitre.PP: string;
begin
   result := DelEnd ( inf, 5 ) + '';
end;

function TVnaitre.RadPres(P: integer): string;
begin
   case P of
      1, 2 : result := DelEnd ( inf, 4 ) + 'i';
      3 : result := DelEnd ( inf, 3 );
      else result := DelEnd ( inf, 4 ) + 'iss';
      end;
end;

function TVnaitre.RadPs: string;
begin
   result := DelEnd ( inf, 4 ) + 'qu';
end;

{ TVcroitre }

function TVcroitre.IndPs(P: integer): string;
   const D : array [1..6] of shortstring =
      ('s', 's', 't', 'mes', 'tes', 'rent');
begin
   result := RD ( radps, D[P] );
end;

function TVcroitre.PP: string;
begin
   result := RadPs;
end;

function TVcroitre.RadPres(P: integer): string;
begin
   if P < 4
      then result := DelEnd ( inf, 3 )
      else result := DelEnd ( inf, 4 ) + 'iss';
end;

function TVcroitre.RadPs: string;
begin
   result := DelEnd ( inf, 5 ) + '';
end;

{ TVcroire }

function TVcroire.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsU[P] );
end;

function TVcroire.PP: string;
begin
   result := radPs + 'u';
end;

function TVcroire.RadPres(P: integer): string;
begin
   case P of
      4, 5 : result := DelEnd ( inf, 3 ) + 'y';
      else result := DelEnd ( inf, 2 )
      end;
end;

function TVcroire.RadPs: string;
begin
   result := DelEnd ( inf, 4 );
end;

{ TVboire }

function TVboire.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsU[P] );
end;

function TVboire.PP: string;
begin
   result := radPs + 'u';
end;

function TVboire.RadPres(P: integer): string;
begin
   case P of
      1..3 : result := DelEnd ( inf, 2 );
      4, 5 : result := DelEnd ( inf, 4 ) + 'uv';
      6 :  result := Delend ( inf, 2 ) + 'v';
      end;
end;

function TVboire.RadPs: string;
begin
   result := DelEnd ( inf, 4 );
end;

function TVboire.SubjPres(P: integer): string;
begin
   case P of
      4, 5 : result := RadPres ( 4 ) + DesSubjPres[P];
      else result := inherited SubjPres ( P );
      end;
end;

{ TVclore }

function TVclore.IndPres(P: integer): string;
begin
  if RadPres ( P ) > ''
     then result := inherited IndPres ( P )
     else result := '';
end;

function TVclore.IndPs(P: integer): string;
begin
   result := '';
end;

function TVclore.PP: string;
begin
   result := DelEnd ( inf, 2 ) + 's'
end;

function TVclore.RadPres(P: integer): string;
begin
   case P of
     1, 2 : result := DelEnd ( inf, 2 );
     3 : result := DelEnd ( inf, 3 ) + '';
     4, 5 : if inf = 'enclore'
        then result := DelEnd ( inf, 2 ) + 's'
        else result := '';
     6 : result := DelEnd ( inf, 2 ) + 's';
     end;
end;

{ TVsoudre }

function TVsoudre.IndPs(P: integer): string;
begin
   result := RD ( RadPs, DesPsU[P] );
end;

function TVsoudre.PP: string;
begin
   result := RadPs + 'u';
end;

function TVsoudre.RadPres(P: integer): string;
begin
   if P < 4
      then result := DelEnd ( inf, 3 )
      else result := DelEnd ( inf, 4 ) + 'lv';
end;

function TVsoudre.RadPs: string;
begin
   result := DelEnd ( inf, 4 ) + 'l';
end;

{ TVcoudre }

function TVcoudre.RadPres(P: integer): string;
begin
   if P < 4
      then result := inherited RadPres ( P )
      else result := DelEnd ( inf, 3 ) + 's';
end;

function TVcoudre.RadPs: string;
begin
   result := RadPres ( 4 );
end;

{ TVmoudre }

function TVmoudre.IndPs(P: integer): string;
begin
   Result := RD ( RadPs, DesPsU[P] );
end;

function TVmoudre.RadPres(P: integer): string;
begin
   if P < 4
      then result := inherited RadPres ( P )
      else result := DelEnd ( inf, 3 ) + 'l';
end;

function TVmoudre.RadPs: string;
begin
   result := RadPres ( 4 );
end;

{ TVsuivre }

function TVsuivre.RadPres(P: integer): string;
begin
   if P < 4
      then result := DelEnd ( inf, 3 )
      else result := Delend ( inf, 2 );
end;

{ TVlire }

function TVlire.IndPs(P: integer): string;
begin
   result := RD ( RadPs, DesPsU[P] );
end;

function TVlire.PP: string;
begin
   result := RadPs + 'u';
end;

function TVlire.RadPres(P: integer): string;
begin
   result := DelEnd ( inf, 2 );
   if P > 3
     then  result := result + 's';
end;

function TVlire.RadPs: string;
begin
   result := DelEnd ( inf, 3 );
end;

{ TVconfire }

function TVconfire.PP: string;
begin
   result := RadPs + 'it';
end;

function TVconfire.RadPres(P: integer): string;
begin
   if P < 4
      then result := DelEnd ( inf, 2 )
      else result := DelEnd ( inf, 2 ) + 's';
end;

function TVconfire.radps: string;
begin
   result := DelEnd ( inf, 3 );
end;

{ TVcourir }

function TVcourir.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsU[P] );
end;

function TVcourir.PP: string;
begin
   result := RadPs + 'u';
end;

function TVcourir.RadFut: string;
begin
   result := DelEnd ( inf, 2 ) + 'r';
end;

{ TVclure }

function TVclure.IndPs(P: integer): string;
begin
   result := RD ( radps, DesPsU[P] );
end;

function TVclure.PP: string;
begin
   result := RadPs + 'u';
end;

function TVclure.radps: string;
begin
   result := DelEnd ( inf, 3 );
end;

{ TVpaitre }

function TVpaitre.IndPs(P: integer): string;
begin
   if inf = 'patre'
      then result := ''
      else result := radPs + DesPsU[P];
end;

function TVpaitre.PP: string;
begin
   if inf = 'patre'
      then result := ''
      else result := RadPs + 'u';
end;

function TVpaitre.RadPs: string;
begin
   result := Delend ( inf, 5 );
end;

{ TVouvrir }

function TVouvrir.IndPs(P: integer): string;
begin
   result := RD ( RadPs, DesPsI[P] );
end;

function TVouvrir.PP: string;
begin
   result := DelEnd ( inf, 3 ) + 'ert';
end;

{ TVrire }

function TVrire.RadPs: string;
begin
   Result := DelEnd ( inf, 3 );
end;

{ TVvivre }

function TVvivre.IndPs(P: integer): string;
begin
   result := RadPs + DesPsU[P] ;
end;

function TVvivre.PP: string;
begin
   result := RadPs + 'u';
end;

function TVvivre.RadPs: string;
begin
   result := DelEnd ( inf, 4 ) + 'c';
end;

{ TValler }

function TValler.IndPres(P : integer): string;
   const F : array[1..6] of shortstring =
      ('vais', 'vas', 'va', 'allons', 'allez', 'vont');
begin
   result := F[P];
end;

function TValler.RadFut: string;
begin
   result := DelEnd ( inf, 5 ) + 'ir';
end;

function TValler.SubjPres(P: integer): string;
   var R : string;
begin
   case P of
      4, 5 : R := 'all';
      else R := 'aill'
      end;
   Result := R + DesSubjPres[P];
end;

{ TVeter }

function TVeter.RD ( R, D : string ) : string;
   var p : integer;
begin
   // reprer la dernire occurence de e dans le radical
   p := PosRev ( 'e', R);
   // transformer en  si l'initiale de D est e.
   if (D[1] = 'e') and ( LastChar(D) <> 'z' )
      then R[p] := '';
   result := R + D;
end;

{ TVebrer }

function TVebrer.RD(R, D: string): string;
   var p : integer;
begin
   // reprer la dernire occurence de  dans le radical
   p := PosRev ( '', R);
   // transformer en  si l'initiale de D est e.
   if (D[1] = 'e') and ( LastChar(D) <> 'z' )
      then R[p] := '';
   result := R + D;
end;

{ TVettell }

function TVettell.RD(R, D: string): string;
begin
   // transformer en  si l'initiale de D est e.
   if (D[1] = 'e') and ( LastChar(D) <> 'z' )
      then R := R + LastChar( R );
   result := R + D;
end;

{ TVyer }

function TVyer.RD(R, D: string): string;
begin
   // transformer en i si l'initiale de D est e.
   if (D[1] = 'e') and ( LastChar(D) <> 'z' ) then
         R := DelEnd ( R, 1 ) + 'i';
   result := R + D;
end;

{ TVvenir }

function TVvenir.circonflexe: string;
begin
   result := DelEnd ( inf, 4 ) + 'n';
end;

function TVvenir.IndPs(P: integer): string;
   const D : array [1..6] of shortstring =
   ('ins', 'ins', 'int', 'nmes', 'ntes', 'inrent');
begin
   result := DelEnd( inf, 4) + D[P];
end;

function TVvenir.PP: string;
begin
   result := DelEnd ( inf, 2 ) + 'u';
end;

function TVvenir.radfut: string;
begin
   result := DelEnd ( inf, 4 ) + 'iendr';
end;

function TVvenir.RadPres(P: integer): string;
begin
   case P of
     1..3 : result := DelEnd ( inf, 4 ) + 'ien';
     4, 5 : result := DelEnd ( inf, 2 );
     6 : result := DelEnd ( inf, 4 ) + 'ienn';
     else result := '';
   end;
end;

{ TVtaire }

function TVtaire.IndPs(P: integer): string;
begin
   result := RadPs + DesPsU [ P ];
end;

function TVtaire.RadPs: string;
begin
   result := delEnd ( inf, 4 );
end;

{ flexion du nom }

function PlurielNom ( F : string ) : string;
begin
   if Pos ( LastChar(F), 'xsz' ) = 0 then
      begin
         if (islast(F, 'eu')) or (islast(F, 'au'))
           then result := F + 'x'
         else if (islast(F, 'al'))
            then result := delEnd(F, 1) + 'ux'
         else result := F + 's'
      end
      else result := F;
end;

initialization

ListeEtre := TStringList( ExtractWords ( AuxEtre ) );
ListeEtre.Sorted := true;

ListeEter := TStringList ( ExtractWords ( Eter ) );
ListeEter.Sorted := true;

finalization

ListeEtre.Free;
ListeEter.Free;

end.
