unit UFlexions;

{Arcus - Extraction du lexique d'un texte latin.

Copyright (C) 2001 Y. Ouvrard.

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le
modifier conformment aux dispositions de la Licence Publique Gnrale GNU,
telle que publie par la Free Software Foundation ; version 2 de la licence,
ou encore ( votre choix) toute version ultrieure.
Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE
GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION
A UN OBJET PARTICULIER.
Pour plus de dtail, voir la Licence Publique Gnrale GNU .
Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme
temps que ce programme ; si ce n'est pas le cas, crivez  la
Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
Pour tout contact avec les auteurs : yves.ouvrard@collatinus.com}

interface

uses Classes, ULex, UDes, UDModele, UIrreg;

type

  TCaseModele = Class (TObject)
      graphie : TStringList;
      def : TDef;
      constructor Create; overload;
      constructor Create (G : String; D : TDef); overload;
    end;

  TIrregsK = Class (TList)
    procedure Trie;
  private
    function GetCaseI(index: integer): TCaseModele;
      public
      property CaseI[index : integer] : TCaseModele read GetCaseI ;
      function Forme (F : integer) : String;
    end;

  TListeKIrregs = Class (TStringlist)
    constructor Calcule;
    private
    procedure Ajoute ( D : TDonneesIrreg ; I : String);
    public
    function IrregsK (K : String) : TIrregsK;
    end;

  TModele = Class (TList)
    Nom : String;
    Numero : integer;
    Constructor Create (N : String);
    private
    function GetCaseM(index: integer): TCaseModele;
    public
      procedure ajoute (G : String ; D : TDef);
                     // graphie, Flags-morpho, radical n
      property CaseM [index : integer] : TCaseModele read GetCaseM ;
      function IndexCase (D : TDef) : integer;
      function Forme (K : String; D : TDonneesLexicales; Fl : integer) : String;
      function Radical1 (K : String) : String;
      procedure Trie;
      // test dbogage
      procedure liste;
      procedure Tableau (K : String; D : TDonneesLexicales);
    end;

  TModeles = Class (TList)
    constructor Calcule;
    private
    function GetModele(index: integer): TModele;
    procedure SetModele(index: integer; const Value: TModele);
    public
    property Modele [index : integer] : TModele
      read GetModele write SetModele; Default;
    end;

  Var Modeles : TModeles;
      KIrregs : TListeKIrregs;

implementation

Uses SysUtils, Forms, dialogs, utiles, UListes;

{ TModele }

procedure TModele.ajoute(G: String; D : TDef);
  var P : integer;
      C : TCaseModele;
begin
  // ajoute au modle la graphie G de Flags T et de radical N R

  p := IndexCase (D);

  // si Def existe : lui ajouter G
  if p >= 0
    then CaseM[p].Graphie.Add (Rebours(G))
  // sinon, ajouter Def, puis lui affecter G
    else
      begin
        C := TCaseModele.Create;
        C.Def := D;
        Add(C);
        Ajoute (G, D);
      end;
end;

constructor TModele.Create (N : String);
begin
  inherited Create;
  Nom := N;
end;

function TModele.Forme(K : string; D : TDonneesLexicales; Fl : integer): String;
  var i : integer;
      Irreg : TIrregsK;
begin
  Result := '';
   // chercher dans les irrguliers
   Irreg := Kirregs.IrregsK(K);
   if (Irreg <> nil)
     then Result := Irreg.Forme (Fl);
   if Result > ''
     then exit;

   // Chercher la morpho dans le modle du K
  for i := 0 to Count - 1 do
    if CaseM[i].Def.Flags = Fl
      then with CaseM[i] do
        begin
          // ShowMessage( intToStr(Def.RadicalNumero));
          Case Def.RadicalNumero of
            1 : 
              case Numero of
                5..8 : Result := D.Radical2 + graphie.commatext;
                else Result := Radical1 (K) + graphie.commatext;
                end;
            2 : Result := D.Radical2 + graphie.CommaText;
            3 : Result := D.Radical3 + graphie.CommaText;
            end;
          break;
        end;
end;

function TModele.GetCaseM(index: integer): TCaseModele;
begin
  Result := items[index];
end;

function Compare (P1, P2 : Pointer) : integer;
    var M1, M2 : TCaseModele;
    begin
      M1 := TCaseModele (P1);
      M2 := TCaseModele (P2);
      if abs(M1.Def.Vox) < abs(M2.Def.Vox) then Result := -1
      else if abs(M1.Def.Vox) > abs(M2.Def.Vox) Then Result := 1
      else
        begin
          if M1.Def.Modus < M2.Def.Modus then Result := -1
          else if M1.Def.Modus > M2.Def.Modus Then Result := 1
          else
            begin
              if M1.Def.tempus < M2.Def.tempus then Result := -1
              else if M1.Def.tempus > M2.Def.tempus Then Result := 1
              else
                begin
                  if M1.Def.genus < M2.Def.genus then Result := -1
                  else if M1.Def.genus > M2.Def.genus Then Result := 1
                  else
                    begin
                      if M1.Def.numerus < M2.Def.numerus then Result := -1
                      else if M1.Def.numerus > M2.Def.numerus Then Result := 1
                      else
                        begin
                          if M1.Def.persona < M2.Def.persona then Result := -1
                          else if M1.Def.persona > M2.Def.persona Then Result := 1
                          else
                            begin
                              if M1.Def.casus < M2.Def.casus then Result := -1
                              else if M1.Def.casus > M2.Def.casus Then Result := 1
                              else result := 0;
                            end;
                        end;
                    end;
                end;
            end;
    end;
  end;


function TModele.IndexCase(D : TDef): integer;
  var // Haut, Bas : integer;
      i : integer;
{  function Trouve (D : TDef; var H, B : integer) : integer;
      var Milieu : integer;
    begin
      Result := -1;
      if Count = 0
        then exit;
      Milieu := H + ((B - H) div 2);
        Case Compare(D, CaseM[Milieu].Def) of
           1 : if Milieu = B
             then Result := -1
             else Result := Trouve(D, H, Milieu);
           -1 : if Milieu = B
             then Result := 1
             else Result := Trouve(D, Milieu, B);
           0 : Result := Milieu;
          end;
    end;}
begin
  // donne le n de la case de Flags T. sinon, renvoie -1
  {Haut := 0;
  Bas := Count - 1;
  Result := Trouve (D, Haut, Bas);}
  Result := -1;
  for i := 0 to Count - 1
    do if CaseM[i].Def.Flags = D.Flags then
      begin
        Result := i;
        break;
      end;
end;

procedure TModele.liste;
 var i : integer;
begin
  with DModele.StringGrid1 do
    begin
     RowCount := succ (Count);
     Cells[1, 0] := 'dsinences';
     For i := 0 to count - 1 do
        begin
          Cells[0, i + 1] := CaseM[i].def.Graphie(false);
          Cells[1, i + 1] := CaseM[i].graphie.CommaText;
        end;
    end;
  DModele.ShowModal;
end;

function TModele.Radical1(K: String): String;
  var oter : byte;
begin
 Case Numero of
    0, 17, 19 : oter := 1;
    1..4, 9..13, 16, 18, 20, 21, 23, 24, 26 : oter := 2;
    22, 25, 27, 28 : oter := 3;
    else oter := 0;
    end;
 result := copy (K, 1, length(K) - oter );
end;

procedure TModele.Tableau(K : String; D: TDonneesLexicales);
  var i : integer;
      Irreg : TIrregsK;
begin
  // affiche la flexion d'un mot;
  with DModele.StringGrid1 do
    begin
      RowCount := succ(Count);
      Cells[1, 0] := 'formes';
      if D.Modele = 29 then
        begin
          Irreg := KIrregs.IrregsK (K);
          Irreg.Trie;
          if irreg <> nil then
            begin
            RowCount := succ (Irreg.count);
            For i := 0 to Irreg.Count - 1 do
              begin
                Cells[0, i + 1] := Irreg.CaseI[i].Def.Graphie(false);
                Cells[1, i + 1] := Irreg.CaseI[i].Graphie.CommaText;
              end;
            end;
        end
        else For i := 0 to count - 1 do
          begin
            Cells[0, i + 1] := CaseM[i].def.Graphie(false);
            Cells[1, i + 1] := Forme(K, D, CaseM[i].Def.Flags);
          end;
    end;
  DModele.ShowModal;
end;

procedure TModele.Trie;
begin
  Sort (Compare);
end;

{ TModeles }

constructor TModeles.Calcule;
  var i, iDes, iDef : integer;
begin
  inherited Create;
  Clear;
  for i := 0 to 30 do
     // crer le modle
    begin
      add(TModele.Create(ListeModeles[i]));
      Modele[i].Numero := i;
    end;

  for iDes := 1 to Desinences.Count - 1 do
    for iDef := 0 to Desinences.Definitions (iDes).count - 1 do
        begin
          Modele[Desinences.Definition (iDes, iDef).Modele].Ajoute (
            Desinences[iDes], Desinences.Definition(iDes, iDef));
        end;
  for i := 0 to 30 do Modele[i].Trie;
end;

function TModeles.GetModele(index: integer): TModele;
begin
  result := Items[index];
end;

procedure TModeles.SetModele(index: integer; const Value: TModele);
begin
  Items[index] := Value;
end;

{ TCaseModele }

constructor TCaseModele.Create;
begin
  inherited Create;
  Graphie := TStringList.Create;
end;

constructor TCaseModele.Create(G: String; D: TDef);
begin
  Create;
  Graphie.Add(G);
  Def := D;
end;

{ TIrregsK }

function TIrregsK.Forme(F: integer): String;
  var i : integer;
begin
  result := '';
  for i := 0 to count - 1
    do if CaseI[i].Def.Flags = F
      then Result := CaseI[i].graphie.commatext;
end;

function TIrregsK.GetCaseI(index: integer): TCaseModele;
begin
  result := Items[index];
end;

procedure TIrregsK.Trie;
begin
  Sort (Compare);
end;

{ TListeKIrregs }

procedure TListeKIrregs.Ajoute(D: TDonneesIrreg; I: String);
  var p : integer;
begin
  // ajoute I, Def dans la liste des formes irrgulires de D.Canon
  if Find (D.Canon, p)
    then IrregsK(D.Canon).Add (TCaseModele.Create(I, D))
    else
      begin
        AddObject(D.Canon, TIrregsK.Create);
        Ajoute (D, I);
      end;
end;

constructor TListeKIrregs.Calcule;
  var iChaine, iDef : integer;
begin
  // partir de la liste d'Irregs;
  // pour chaque forme irreg :
  inherited Create;
  Sorted := True;

  for iChaine := 0 to Irregs.count - 1
   do for iDef := 0 to Irregs.Definitions(iChaine).count - 1
     do Ajoute(Irregs.Definition(iChaine, iDef), Irregs[iChaine]);
end;

function TListeKIrregs.IrregsK(K: String): TIrregsK;
  var p : integer;
begin
  if Find (K, p)
    then Result := TIrregsK (Objects[p])
    else Result := nil;
end;

initialization
  Modeles := TModeles.Calcule;
  KIrregs := TListeKIrregs.Calcule;

finalization
  Modeles.Free;
  KIrregs.Free;
end.
